@@ -22,14 +22,18 @@
 # Standard library imports.
 from StringIO import StringIO
 from zipfile import ZipFile, ZIP_DEFLATED
+import json
 
 # Django imports.
 from django.contrib.auth.decorators import permission_required
 from django.contrib import messages
+from django.core.exceptions import ValidationError
 from django.core.urlresolvers import reverse, reverse_lazy
+from django.db.models import Q
+from django.db.models.deletion import Collector
 from django.http import HttpResponse
 from django.shortcuts import render_to_response, get_object_or_404
-from django.views.generic import TemplateView, DetailView, CreateView, UpdateView, DeleteView
+from django.views.generic import TemplateView, DetailView, CreateView, UpdateView, DeleteView, View
 
 # Third-party application imports.
 from braces.views import MultiplePermissionsRequiredMixin, SetHeadlineMixin
@@ -62,6 +66,43 @@ class RedirectToNextMixin(object):
         return self.request.GET.get(self.next_parameter, super(RedirectToNextMixin, self).get_success_url())
 
 
+class RelatedItemsMixin(object):
+    """
+    View mixin that adds related items of a referenced model object to context
+    data in form of a nested list, including the reference model object as the
+    first element of a list.
+
+    The context data will be passed using the "related_items" key.
+
+    This data can be used in the template by passing it through the
+    unordered_list template tag.
+
+    The reference object is accessed via "object" property of calling view
+    (i.e. self.object).
+
+    For more details on implementation, see:
+
+    RelatedCollectorMixin.get_dependant_objects_representation method
+    """
+
+    def get_context_data(self, **kwargs):
+        """
+        Adds the related items of a reference model object to context data.
+
+        Returns:
+          Context data.
+        """
+
+        # Set the context using the parent class.
+        context = super(RelatedItemsMixin, self).get_context_data(**kwargs)
+
+        # Add to context the nested list of string representations of related
+        # items.
+        context["related_items"] = self.object.get_dependant_objects_representation()
+
+        return context
+
+
 class IndexView(MultiplePermissionsRequiredMixin, TemplateView):
     """
     Custom view used for rendering the index page.
@@ -136,6 +177,9 @@ class ProjectView(MultiplePermissionsRequiredMixin, DetailView):
         # Add the (location, entities) tuples to context.
         context['location_entities'] = location_entities
 
+        # Add all project communications to context.
+        context['communications'] = self.object.get_project_communications_summary()
+
         # Finally return the context.
         return context
 
@@ -334,7 +378,7 @@ class ProjectUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissio
         return "Update project %s" % self.object.name
 
 
-class ProjectDeleteView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
+class ProjectDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
     """
     View for deleting a project.
     """
@@ -418,7 +462,7 @@ class LocationUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissi
         return "Update location %s" % self.object.name
 
 
-class LocationDeleteView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
+class LocationDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
     """
     View for deleting a location.
     """
@@ -486,13 +530,13 @@ class EntityCreateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermission
         project_id = self.request.GET.get("project", None)
         if project_id:
             form.fields["project"].queryset = Project.objects.filter(pk=project_id)
-            form.fields["project"].widget.attrs["readonly"] = True
+            form.fields["project"].empty_label = None
 
         # Limit the location selection if required.
         location_id = self.request.GET.get("location", None)
         if location_id:
             form.fields["location"].queryset = Location.objects.filter(pk=location_id)
-            form.fields["location"].widget.attrs["readonly"] = True
+            form.fields["location"].empty_label = None
 
         return form
 
@@ -535,7 +579,7 @@ class EntityUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermission
         return "Update entity %s" % self.object.name
 
 
-class EntityDeleteView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
+class EntityDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
     """
     View for deleting an entity.
     """
@@ -613,7 +657,7 @@ class InterfaceCreateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermiss
         entity_id = self.request.GET.get("entity", None)
         if entity_id:
             form.fields["entity"].queryset = Entity.objects.filter(pk=entity_id)
-            form.fields["entity"].widget.attrs["readonly"] = True
+            form.fields["entity"].empty_label = False
 
         return form
 
@@ -691,7 +735,7 @@ class InterfaceUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermiss
         return "Update interface %s" % self.object.name
 
 
-class InterfaceDeleteView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
+class InterfaceDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
     """
     View for deleting an interface.
     """
@@ -884,7 +928,7 @@ class CommunicationUpdateView(RedirectToNextMixin, SetHeadlineMixin, MultiplePer
         return "Update communication %s" % self.object
 
 
-class CommunicationDeleteView(RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
+class CommunicationDeleteView(RelatedItemsMixin, RedirectToNextMixin, SetHeadlineMixin, MultiplePermissionsRequiredMixin, DeleteView):
     """
     View for deleting an communication.
     """
@@ -965,3 +1009,127 @@ def project_diagram(request, pk):
 
     # Return the response object.
     return response
+
+
+class SearchView(MultiplePermissionsRequiredMixin, TemplateView):
+    """
+    Custom view used for rendering the search (results) page.
+    """
+
+    template_name = 'conntrackt/search.html'
+
+    # Required permissions.
+    permissions = {
+        "all": ("conntrackt.view",),
+        }
+
+    # Raise authorisation denied exception for unmet permissions.
+    raise_exception = True
+
+    def get_context_data(self, **kwargs):
+        """
+        Returns the context data that should be used for rendering of the
+        template.
+
+        Adds context objects:
+          - 'entities', which is a list of entities that had the search term in
+            their name or description.
+          - 'projects', which is a list of entities that had the search term in
+            their name or description.
+          - 'search_term', which is a string of previous query that brought the
+            user to page (if any). The term will be stripped from leading and
+            trailing spaces/tabs.
+        """
+
+        # Set the context using the parent aclass.
+        context = super(SearchView, self).get_context_data(**kwargs)
+
+        # Retrieve the search term, and strip it if it was provided.
+        search_term = self.request.GET.get("q", None)
+        if search_term:
+            search_term = search_term.strip()
+
+        # Do not allow empty searches.
+        if search_term == "":
+            messages.error(self.request, "Search query is not allowed to be empty.", extra_tags="alert alert-error")
+        # Set-up the context objects if search was sent. Otherwise empty search
+        # page will be shown.
+        elif search_term is not None:
+            context['search_term'] = search_term
+            context['entities'] = Entity.objects.search(search_term)
+            context['projects'] = Project.objects.search(search_term)
+
+        return context
+
+
+class APISearchView(MultiplePermissionsRequiredMixin, View):
+    """
+    API view implementing search for entities and projects that match the
+    provided search term.
+
+    The output generated by the view uses JSON. The result will include a list
+    of matched items, where each item is a dictionary with the following keys:
+
+      - name (name of the matched item)
+      - project (project to which the item belongs)
+      - type (type of the matched item)
+      - url (URL towards the matched item)
+    """
+
+    # Required permissions.
+    permissions = {
+        "all": ("conntrackt.view",),
+        }
+
+    # Raise authorisation denied exception for unmet permissions.
+    raise_exception = True
+
+    def get(self, request, search_term=""):
+        """
+        Implements response handling for a GET request.
+        """
+
+        # Retrieve the search term, and strip it if it was provided.
+        if search_term:
+            search_term = search_term.strip()
+
+        # Set-up a list that will contain found items.
+        items = []
+
+        # Fetch the maximum number of items that should be returned.
+        limit = int(request.GET.get("limit", 0))
+        if limit < 0:
+            raise ValidationError("Limit may not be a negative value.")
+
+        # Don't perform search with empty search term.
+        if search_term != "":
+
+            # Run the search on entities and projects.
+            entities = Entity.objects.search(search_term).select_related("project")
+            projects = Project.objects.search(search_term)
+
+            # If maximum number of items was provided, narrow-down the results.
+            if limit > 0:
+                entities = entities[:limit]
+                projects = projects[:limit]
+
+            # Add found entities.
+            for entity in entities:
+                items.append({"name": entity.name,
+                              "project": entity.project.name,
+                              "type": "entity",
+                              "url": entity.get_absolute_url()})
+
+            # Add found projects.
+            for project in projects:
+                items.append({"name": project.name,
+                              "project": project.name,
+                              "type": "project",
+                              "url": project.get_absolute_url()})
+
+        # Generate the JSON response.
+        content = json.dumps(items)
+        response = HttpResponse(content, mimetype="application/json")
+
+        # Return the response.
+        return response
